## Disparity Estimation Networks for Aerial and High-Resolution Remote Sensing Images: A Review

by [Roger Marí](https://scholar.google.com/citations?user=TgpSmIsAAAAJ&hl), Centre Borelli, ENS Paris-Saclay, 2022.

This work is dedicated to the evaluation of deep learning architectures for estimating disparity from an input stereo pair of high resolution aerial or satellite images. We use models trained using a recently proposed stereo dense matching benchmark for aerial imagery, based on the ISPRS Vaihingen dataset.

---
## Setup and dependencies

The main dependencies are Python 3.7 and PyTorch 1.6.0+.

You can set up a conda environment named `DispNets` to run the code using the following command:
```
conda init && bash -i setup_DispNets_env.sh
```
---
## How to test the networks

Example call to test PSM net, from [*Pyramid Stereo Matching Network* (CVPR 2018)](https://openaccess.thecvf.com/content_cvpr_2018/papers/Chang_Pyramid_Stereo_Matching_CVPR_2018_paper.pdf)
```shell
(DispNets) $ python3 test.py --rectify --model PSM --model_path data/pretrained_models/finetune_PSMnet.tar
```
Example call to test HSM net, from [*Hierarchical Deep Stereo Matching on High-resolution Images* (CVPR 2019)](https://openaccess.thecvf.com/content_CVPR_2019/papers/Yang_Hierarchical_Deep_Stereo_Matching_on_High-Resolution_Images_CVPR_2019_paper.pdf)
```shell
(DispNets) $ python3 test.py --rectify --model HSM --model_path data/pretrained_models/finetune_HSMnet.tar
```
---
## Related articles

If you find this code useful do not forget to cite these articles. This work is just a recycled and compact version of the codes below:

#### 1) [\[Wu et al., 2021\] *A New Stereo Dense Matching Benchmark Dataset for Deep Learning*. ISPRS Archives 2021.](https://www.int-arch-photogramm-remote-sens-spatial-inf-sci.net/XLIII-B2-2021/405/2021/isprs-archives-XLIII-B2-2021-405-2021.pdf)

 This article presents a dataset to estimate disparity from stereo pairs of aerial images, which was used to finetune the disparity estimation networks evaluated in this work. [>> Link to the original code here <<](https://github.com/whuwuteng/benchmark_ISPRS2021)

```
@article{wu2021new,
  title={A New Stereo Dense Matching Benchmark Dataset for Deep Learning},
  author={Wu, T and Vallet, B and Pierrot-Deseilligny, M and Rupnik, E},
  journal={The International Archives of Photogrammetry, Remote Sensing and Spatial Information Sciences},
  volume={43},
  pages={405--412},
  year={2021}
}
```

#### 2) [\[Chang & Chen, 2018\] *Pyramid Stereo Matching Network*. CVPR 2018.](https://openaccess.thecvf.com/content_cvpr_2018/papers/Chang_Pyramid_Stereo_Matching_CVPR_2018_paper.pdf)

This one of the disparity estimation networks evaluated in this work. [>> Link to the original code here <<](https://github.com/JiaRenChang/PSMNet)

```
@inproceedings{chang2018pyramid,
  title={Pyramid Stereo Matching Network},
  author={Chang, Jia-Ren and Chen, Yong-Sheng},
  booktitle={Proceedings of the IEEE Conference on Computer Vision and Pattern Recognition},
  pages={5410--5418},
  year={2018}
}
```

#### 3) [\[Yang et al., 2019\] *Hierarchical Deep Stereo Matching on High-resolution Images*. CVPR 2019.](https://openaccess.thecvf.com/content_CVPR_2019/papers/Yang_Hierarchical_Deep_Stereo_Matching_on_High-Resolution_Images_CVPR_2019_paper.pdf)

This one of the disparity estimation networks evaluated in this work. [>> Link to the original code here <<](https://github.com/gengshan-y/high-res-stereo)

```
@inproceedings{yang2019hierarchical,
  title={Hierarchical Deep Stereo Matching on High-resolution Images},
  author={Yang, Gengshan and Manela, Joshua and Happold, Michael and Ramanan, Deva},
  booktitle={Proceedings of the IEEE/CVF Conference on Computer Vision and Pattern Recognition},
  pages={5515--5524},
  year={2019}
}
```
